from django.test import TestCase, override_settings


@override_settings(ROOT_URLCONF="django_early_return.tests.urls")
class EarlyReturnTestCase(TestCase):
    def test_early_return_middleware_is_installed_and_work_as_expected(self):
        r = self.client.get("/raise_forbidden/")
        self.assertEqual(r.status_code, 403)
        self.assertEqual(r.content, b"access denied")



from django import http
from django.urls import path

from django_early_return import EarlyReturn

def raise_forbidden(request):
    raise EarlyReturn(http.HttpResponseForbidden('access denied'))

urlpatterns = [
    path('raise_forbidden/', raise_forbidden),
]


from typing import Callable

from django import http


class EarlyReturn(Exception):
    """
    Exception which views may raise, when using EarlyReturnMiddleware, to short-circuit view processing
    """

    def __init__(self, response: http.HttpResponse):
        self.response = response


class EarlyReturnMiddleware:
    """
    Middleware which lets views/view decorators raise EarlyReturn to short-circuit processing
    """

    def __init__(self, get_response: Callable[[http.HttpRequest], http.HttpResponse]):
        self.get_response = get_response

    def __call__(self, request: http.HttpRequest):
        return self.get_response(request)

    def process_exception(self, request: http.HttpRequest, exception: Exception):
        if isinstance(exception, EarlyReturn):
            return exception.response



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django_early_return",
    version="0.3.0",
    author="Alex Fischer",
    author_email="alex@quadrant.net",
    description="Allows view code to exit early and return a given HttpResponse by raising an exception",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="TODO",
    packages=["django_early_return", "django_early_return.tests"],
    package_data={
        "django_early_return": ["py.typed"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=["Django>=2.2,<5"],
)
